#ifndef BLE_APH_SERVICE_H__
#define BLE_APH_SERVICE_H__

#include <stdint.h>
#include <stdbool.h>

#include "ble.h"
#include "ble_srv_common.h"

// The uuid of service and characteristics
#define APH_SERVICE_BASE {0x71, 0x3D, 0x00, 0x00, 0x50, 0x3E, 0x4C, 0x75, 0xBA, 0x94, 0x31, 0x48, 0xF1, 0x8D, 0x94, 0x1E}
#define APH_SERVICE_BASE_REV {0x1E, 0x94, 0x8D, 0xF1, 0x48, 0x31, 0x94, 0xBA, 0x75, 0x4C, 0x3E, 0x50, 0, 0, 0x3D, 0x71}
#define APH_TX_BASE {0x71, 0x3D, 0x00, 0x03, 0x50, 0x3E, 0x4C, 0x75, 0xBA, 0x94, 0x31, 0x48, 0xF1, 0x8D, 0x94, 0x1E}
#define APH_RX_BASE {0x71, 0x3D, 0x00, 0x02, 0x50, 0x3E, 0x4C, 0x75, 0xBA, 0x94, 0x31, 0x48, 0xF1, 0x8D, 0x94, 0x1E}
#define APH_SERVICE_UUID 0x0000
#define APH_TX_UUID 0x0003
#define APH_RX_UUID 0x0002

/**@brief Custom Service event type. */
typedef enum {
    BLE_CUS_EVT_NOTIFICATION_ENABLED,
    /**< Custom value notification enabled event. */
    BLE_CUS_EVT_NOTIFICATION_DISABLED,
    /**< Custom value notification disabled event. */
    BLE_CUS_EVT_DISCONNECTED,
    BLE_CUS_EVT_CONNECTED
} ble_cus_evt_type_t;

/*
 * @details Handles all events from the BLE stack of interest to the Battery Service.
 *
 * @note
 *
 * @param[in]   p_cus      Custom Service structure.
 * @param[in]   p_ble_evt  Event received from the BLE stack.
 */
void ble_cus_on_ble_evt (ble_evt_t const *p_ble_evt, void *p_context);
/**@brief   Macro for defining a ble_hrs instance.
 *
 * @param   _name   Name of the instance.
 * @hideinitializer
 */
#define BLE_CUS_DEF(_name) \
static ble_cus_t _name; \
NRF_SDH_BLE_OBSERVER (_name ## _obs,                                                                 \
			BLE_HRS_BLE_OBSERVER_PRIO, \
ble_cus_on_ble_evt, &_name)

/**@brief Custom Service event. */
typedef struct {
    ble_cus_evt_type_t evt_type;					  /**< Type of event. */
} ble_cus_evt_t;

// Forward declaration of the ble_cus_t type.
typedef struct ble_cus_s ble_cus_t;

/**@brief Custom Service event handler type. */
typedef void (*ble_cus_evt_handler_t) (ble_cus_t * p_cus,
                                       ble_cus_evt_t * p_evt);
									   
/*        initialization of the service.*/
typedef struct {
    ble_cus_evt_handler_t evt_handler;				  /**< Event handler to be called for handling events in the Custom Service. */
    uint8_t initial_custom_value;					  /**< Initial custom value */
    ble_srv_cccd_security_mode_t custom_value_char_attr_md;	 /**< Initial security level for Custom characteristics attribute */
} ble_cus_init_t;

/**@brief Custom Service structure. This contains various status information for the service. */
struct ble_cus_s {
    ble_cus_evt_handler_t evt_handler;				  /**< Event handler to be called for handling events in the Custom Service. */
    uint16_t service_handle;					  /**< Handle of Custom Service (as provided by the BLE stack). */
    ble_gatts_char_handles_t tx_handles; //msg_packet handle
    ble_gatts_char_handles_t rx_handles; //rx_packet handle
    uint16_t conn_handle;						  /**< Handle of the current connection (as provided by the BLE stack, is BLE_CONN_HANDLE_INVALID if not in a connection). */
    uint8_t uuid_type;
};

/**@brief Function for initializing the Custom Service.
 *
 * @param[out]  p_cus       Custom Service structure. This structure will have to be supplied by
 *                          the application. It will be initialized by this function, and will later
 *                          be used to identify this particular service instance.
 * @param[in]   p_cus_init  Information needed to initialize the service.
 *
 * @return      NRF_SUCCESS on successful initialization of service, otherwise an error code.
 */
ret_code_t ble_cus_init (ble_cus_t * p_cus,
                         const ble_cus_init_t * p_cus_init);
						 
/**@brief Function for updating the custom value.
 *
 * @details The application calls this function when the cutom value should be updated. If
 *          notification has been enabled, the custom value characteristic is sent to the client.
 *
 * @note
 *
 * @param[in]   p_cus          Custom Service structure.
 * @param[in]   Custom value
 *
 * @return      NRF_SUCCESS on success, otherwise an error code.
 */
ret_code_t updateTX (ble_cus_t * p_cus,uint8_t custom_value[5]);

#endif
